@@ -16,13 +16,8 @@
 #include <getopt.h>
 #include <string.h>
 #include <stdio.h>
-#include <fcntl.h>
 #include <errno.h>
 
-#ifdef _WIN32
-#include <io.h>
-#endif
-
 /* options.c */
 extern bool dont_skip;
 extern bool keep_time;
@@ -34,6 +29,6 @@ extern char *root_becomes;
 void process_args(int argc, char **argv);
 
 /* process_tarball.c */
-int process_tarball(FILE *input_file, sqfs_writer_t *sqfs);
+int process_tarball(istream_t *input_file, sqfs_writer_t *sqfs);
 
 #endif /* TAR2SQFS_H */
